﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace CarFire
{
    /// <summary>
    /// This class represents a projectile object that will be spawned whenever a player or a monster fires.
    /// </summary>
    public class Projectile
    {
        //Member Variables
        Map theMap;
        Vector2 velocity;
        Texture2D projectileModel;
        int damage;
        int gridX;
        int gridY;
        //The pixel position should be the pixel position on the map.  When a projectile is drawn
        //these will have to be transformed to the coordinate system that the drawable screen is using.
        int pixelX;
        int pixelY;
   
        /// <summary>
        /// The Constructor for a projectile object.
        /// </summary>
        /// <param name="_currentMap">The map that this character will interact with</param>
        /// <param name="_projectileModel">The model for this projectile</param>
        /// <param name="_velocity">How fast the projectile moves</param>
        /// <param name="_gridX">The starting X position in the map grid</param>
        /// <param name="_gridY">The starting Y position in the map grid</param>
        /// <param name="_pixelX">The absolute X pixel position on the map</param>
        /// <param name="_pixelY"> The absolute Y pixel position on the map</param>
        public Projectile(Map _currentMap,
                            Texture2D _projectileModel,
                            Vector2 _velocity,
                            int _gridX,
                            int _gridY,
                            int _pixelX,
                            int _pixelY)
        {
            theMap = _currentMap;
            projectileModel = _projectileModel;
            velocity = _velocity;
            gridX = _gridX;
            gridY = _gridY;
            pixelX = _pixelX;
            pixelY = _pixelY;
        }
        public void Update()
        {
            //See if something moved onto this projectile.
            if(theMap.IsCellOpen(gridX, gridY))
            {
                //theMap.damageSquare(gridX, gridY, damage);
            }
            //If the projectile will be moving to a new grid position we need to check to see if it is occupied.
            /*
            if ((int)((pixelX + velocity.X) / theMap.gridToPixelRatio) != gridX || (int)((pixelY + velocity.Y) / Map.gridToPixelRatio) != gridY)
            {
                bool open = theMap.IsCellOpen((pixelX + velocity.X) / theMap.gridToPixelRatio, (pixelY + velocity.Y) / Map.gridToPixelRatio);
                //If open just move this projectile there
                //***Map doesn't need to know that this projectile is there because players/monsters are allowed
                // to move into the path of projectiles.
                if (open)
                {
                    pixelX += (int)velocity.X;
                    pixelY += (int)velocity.Y;
                    gridX = pixelX / theMap.gridToPixelRatio;
                    gridY = pixelY / theMap.gridToPixelRatio;
                }
                //If the square isn't open then just damage whatever is there
                //TODO: A projectile must be deleted after this happens
                else
                {
                    //theMap.damageSquare(gridX, gridY, damage);
                }
            }
            */
            //If it is not moving grid positions just increment pixelX and pixelY
            else
            {
                pixelX += (int)velocity.X;
                pixelY += (int)velocity.Y;
            }
            
        }
        /// <summary>
        /// This method will draw a projectile to the screen
        /// </summary>
        /// <param name="spriteBatch"></param>
        /// <param name="topLeftX">The map X pixel position of the topLeft of the display</param>
        /// <param name="topLeftY">The map Y pixel position of the topLeft of the display</param>
        public void Draw(SpriteBatch spriteBatch, int topLeftX, int topLeftY)
        {
            spriteBatch.Draw(projectileModel, new Vector2(topLeftX - pixelX, topLeftY - pixelY), null, Color.White, 0, new Vector2(0f, 0f), 1f, SpriteEffects.None, 0);
        }

        /// <summary>
        /// Basic getters and setters
        /// </summary>
        public int GridX { get { return gridX; } set { gridX = value; } }
        public int GridY { get { return gridY; } set { gridY = value; } }
        public int PixelX { get { return pixelX; } set { pixelX = value; } }
        public int PixelY { get { return pixelY; } set { pixelY = value; } }
        public Map TheMap { get { return theMap; } set { theMap = value; } }
        public int Damage { get { return damage; } set { damage = value; } }
    }
}
